use async_trait::async_trait;
use sea_orm_migration::{prelude::*, schema::*};

#[derive(DeriveMigrationName)]
pub struct Migration;

#[async_trait]
impl MigrationTrait for Migration {
    async fn up(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .create_table(
                Table::create()
                    .table(Nodes::Table)
                    .if_not_exists()
                    .col(pk_auto(Nodes::Id))
                    .col(string(Nodes::Hostname))
                    .col(integer(Nodes::PveId))
                    .col(string(Nodes::PveHost))
                    .col(string(Nodes::AssignedIp))
                    .col(integer(Nodes::AssignedSubnet))
                    .col(boolean(Nodes::IsPermanent).default(false))
                    .to_owned())
            .await
    }

    async fn down(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .drop_table(
                Table::drop()
                    .table(Nodes::Table)
                    .to_owned())
            .await
    }
}

#[derive(DeriveIden)]
enum Nodes {
    Table,
    Id,
    Hostname,
    PveId,
    PveHost,
    AssignedIp,
    AssignedSubnet,
    IsPermanent,
}
