mod migrations;

use std::time::Duration;
use rocket::figment::Figment;
use sea_orm;
use sea_orm_rocket::{Config, Database, Pool};
use async_trait::async_trait;
use rocket::fairing::AdHoc;
use sea_orm::ConnectOptions;
use crate::api::entity::locks::ensure_vmid_lock;

#[derive(Database, Debug)]
#[database("p5x_api")]
pub struct Db(DbPool);

#[derive(Debug, Clone)]
pub struct DbPool {
    pub conn: sea_orm::DatabaseConnection,
}

#[async_trait]
impl Pool for DbPool {
    type Connection = sea_orm::DatabaseConnection;
    type Error = sea_orm::DbErr;

    async fn init(figment: &Figment) -> Result<Self, Self::Error> {
        let config = figment.extract::<Config>().unwrap();
        let mut options: ConnectOptions = config.url.into();

        debug!("max_connections: {} | min: {}", config.max_connections, config.min_connections.unwrap_or_default());

        options
            // .max_connections(config.max_connections as u32)
            // .min_connections(config.min_connections.unwrap_or_default())
            .max_connections(512)
            .min_connections(128)
            .connect_timeout(Duration::from_secs(config.connect_timeout))
            .sqlx_logging(config.sqlx_logging);

        if let Some(idle_timeout) = config.idle_timeout {
            options.idle_timeout(Duration::from_secs(idle_timeout));
        }

        let conn = sea_orm::Database::connect(options).await?;

        ensure_vmid_lock(&conn).await?;  // todo: probably a better place to put this

        Ok(DbPool { conn })
    }

    fn borrow(&self) -> &Self::Connection {
        &self.conn
    }
}

// async fn run_migrations(rocket: Rocket<Build>) -> fairing::Result {
//     let conn = &Db::fetch(&rocket).unwrap().conn;
//
// }

pub fn init() -> AdHoc {
    AdHoc::on_ignite("Setting up database pool", |rocket| async {
        rocket.attach(Db::init())
            .attach(migrations::init())
    })
}
