use rocket::fairing::AdHoc;
use rocket::response::status;
use rocket::serde::json::Json;
use sea_orm_rocket::Connection;
use crate::api;
use crate::api::cluster;
use crate::api::entity::nodes::NodeParams;
use crate::api::services::Services;
use crate::api::util::raise_500;

#[post("/", data = "<input>")]
async fn register_node(
    conn: Connection<'_, api::Db>,
    input: Json<NodeParams>,
) -> Result<Json<NodeParams>, status::Custom<String>> {
    let input = input.into_inner();
    let db = conn.into_inner();
    let svc = Services::build(db).await.map_err(raise_500)?;

    let node: NodeParams = cluster::node::register_node(&svc, &input)
        .await
        .map_err(raise_500)?
        .into();

    Ok(Json(node.into()))
}

#[delete("/<hostname>")]
async fn unregister_node(
    conn: Connection<'_, api::Db>,
    hostname: &str,
) -> Result<Json<serde_json::Value>, status::Custom<String>> {
    let db = conn.into_inner();
    let svc = Services::build(db).await.map_err(raise_500)?;

    cluster::node::unregister_node(&svc, hostname).await.map_err(raise_500)?;

    Ok(Json(serde_json::json!({})))
}

pub(super) fn init() -> AdHoc {
    AdHoc::on_ignite("Routes: /node", |rocket| async {
        rocket.mount("/node", routes![register_node, unregister_node])
    })
}
